#include "structs.h"
#include "Ports.h"
#include "LED.h"
#include <inttypes.h> 
#include <avr/interrupt.h>




typedef void ( *FunctionPointer ) ( void );
volatile FunctionPointer PS2JumpAddress;
volatile uint8_t PS2ReceivedByte;
volatile uint8_t PS2SendByte;
volatile uint8_t PS2SendByteChecksum;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Forward references
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void PS2ReceiveNop();
void PS2ReceiveStart();
void PS2ReceiveBit0();
void PS2ReceiveBit1();
void PS2ReceiveBit2();
void PS2ReceiveBit3();
void PS2ReceiveBit4();
void PS2ReceiveBit5();
void PS2ReceiveBit6();
void PS2ReceiveBit7();
void PS2ReceiveParity();
void PS2ReceiveStop();
void PS2SetReceivedByteToRam();
void PS2StartSend();
void PS2PullDownClock();
void PS2ReleaseClock();
void PS2PullDownData();
void PS2ReleaseData();
void PS2SendBit0();
void PS2SendBit1();
void PS2SendBit2();
void PS2SendBit3();
void PS2SendBit4();
void PS2SendBit5();
void PS2SendBit6();
void PS2SendBit7();
void PS2SendBitParity();
void PS2SendBitStop();
void PS2Send_ReceiveAck();





void PS2Init()
{

	PCMSK1 = (1<<PCINT11); //Enable Pin change inerrupt on PC3 ()PCINT11)
	PCICR = (1<<PCIE1);   //Enable external interrupt for PCINT14:8

	//Reset variables
	DataRead.LastOffset =0;
	DataWrite.MouseCommand = 0;
	PS2ReceivedByte =0;


	for (int i =0; i<=255;i++)
		DataRead.MouseBuffer[i] = 0;

	PS2JumpAddress = PS2ReceiveStart;
}

//Dummy routine for listen disable
void PS2ReceiveNop()
{}




void PS2ReceiveStart()
{
	SetLED2();
	PS2ReceivedByte =0;
	PS2JumpAddress = PS2ReceiveBit0;
}

void PS2ReceiveBit0()
{
	if (PINC & (1<<PS2Data))
		PS2ReceivedByte |= 0x80;
	PS2ReceivedByte = (PS2ReceivedByte >> 1);
	PS2JumpAddress = PS2ReceiveBit1;
}
void PS2ReceiveBit1()
{
	if (PINC & (1<<PS2Data))
		PS2ReceivedByte |= 0x80;
	PS2ReceivedByte = (PS2ReceivedByte >> 1);
	PS2JumpAddress = PS2ReceiveBit2;
}
void PS2ReceiveBit2()
{
	if (PINC & (1<<PS2Data))
		PS2ReceivedByte |= 0x80;
	PS2ReceivedByte = (PS2ReceivedByte >> 1);
	PS2JumpAddress = PS2ReceiveBit3;
}
void PS2ReceiveBit3()
{
	if (PINC & (1<<PS2Data))
		PS2ReceivedByte |= 0x80;
	PS2ReceivedByte = (PS2ReceivedByte >> 1);
	PS2JumpAddress = PS2ReceiveBit4;
}
void PS2ReceiveBit4()
{
	if (PINC & (1<<PS2Data))
		PS2ReceivedByte |= 0x80;
	PS2ReceivedByte = (PS2ReceivedByte >> 1);
	PS2JumpAddress = PS2ReceiveBit5;
}
void PS2ReceiveBit5()
{
	if (PINC & (1<<PS2Data))
		PS2ReceivedByte |= 0x80;
	PS2ReceivedByte = (PS2ReceivedByte >> 1);
	PS2JumpAddress = PS2ReceiveBit6;
}
void PS2ReceiveBit6()
{
	if (PINC & (1<<PS2Data))
		PS2ReceivedByte |= 0x80;
	PS2ReceivedByte = (PS2ReceivedByte >> 1);
	PS2JumpAddress = PS2ReceiveBit7;
}
void PS2ReceiveBit7()
{
	if (PINC & (1<<PS2Data))
		PS2ReceivedByte |= 0x80;
	PS2SetReceivedByteToRam();
	PS2JumpAddress = PS2ReceiveParity;
}



void PS2ReceiveParity()
{
	PS2JumpAddress = PS2ReceiveStop;
}

void PS2ReceiveStop()
{
	PS2JumpAddress = PS2ReceiveStart;
//	ClearLED1();
}




void PS2SetReceivedByteToRam()
{
	DataRead.MouseBuffer[DataRead.LastOffset] = PS2ReceivedByte;
	DataRead.LastOffset++;
}




//++++++++++++++++++++++++++++++++++++++++++
//External PCINT PinChange Interrupt
//++++++++++++++++++++++++++++++++++++++++++
ISR(PCINT1_vect)
{

	SetLED3();
	if ((PINC & (1<<PS2Clock)) == 0)
		PS2JumpAddress();
}





//******************************************************************************************************************
//SEND
//******************************************************************************************************************
void PS2Send(uint8_t bt)
{
	PS2SendByte = bt;

	//Revceiver off
	PS2JumpAddress = PS2ReceiveNop;

	//Prepare checksum
	PS2SendByteChecksum= MakeChecksumOfByte(PS2SendByte);

	// pull down Clock for 100u
	PS2PullDownClock();
	Wait100u();;

	// initialize Hump address
	PS2JumpAddress = PS2SendBit0;
	
	// pulldown data and release clock
	PS2PullDownData();
	PS2ReleaseClock();
	ToggleLED1();
}


//Send states
void PS2SendBit0()
{
	if((PS2SendByte & 0x01) !=0)
		PS2ReleaseData();
	else
		PS2PullDownData();
	PS2SendByte = (PS2SendByte >>1);
	PS2JumpAddress = PS2SendBit1;
}
void PS2SendBit1()
{
	if((PS2SendByte & 0x01) !=0)
		PS2ReleaseData();
	else
		PS2PullDownData();
	PS2SendByte = (PS2SendByte >>1);
	PS2JumpAddress = PS2SendBit2;
}
void PS2SendBit2()
{
	if((PS2SendByte & 0x01) !=0)
		PS2ReleaseData();
	else
		PS2PullDownData();
	PS2SendByte = (PS2SendByte >>1);
	PS2JumpAddress = PS2SendBit3;
}
void PS2SendBit3()
{
	if((PS2SendByte & 0x01) !=0)
		PS2ReleaseData();
	else
		PS2PullDownData();
	PS2SendByte = (PS2SendByte >>1);
	PS2JumpAddress = PS2SendBit4;
}
void PS2SendBit4()
{
	if((PS2SendByte & 0x01) !=0)
		PS2ReleaseData();
	else
		PS2PullDownData();
	PS2SendByte = (PS2SendByte >>1);
	PS2JumpAddress = PS2SendBit5;
}
void PS2SendBit5()
{
	if((PS2SendByte & 0x01) !=0)
		PS2ReleaseData();
	else
		PS2PullDownData();
	PS2SendByte = (PS2SendByte >>1);
	PS2JumpAddress = PS2SendBit6;
}
void PS2SendBit6()
{
	if((PS2SendByte & 0x01) !=0)
		PS2ReleaseData();
	else
		PS2PullDownData();
	PS2SendByte = (PS2SendByte >>1);
	PS2JumpAddress = PS2SendBit7;
}
void PS2SendBit7()
{
	if((PS2SendByte & 0x01) !=0)
		PS2ReleaseData();
	else
		PS2PullDownData();

	PS2JumpAddress = PS2SendBitParity;
}





void PS2SendBitParity()
{
	if((PS2SendByteChecksum & 0x01) !=0)
		PS2PullDownData();
	else
		PS2ReleaseData();

	PS2JumpAddress = PS2SendBitStop;
}

void PS2SendBitStop()
{
	PS2ReleaseData();
	PS2JumpAddress = PS2Send_ReceiveAck;
}

void PS2Send_ReceiveAck()
{
	if ((PINC & (1<<PS2Data)) != 0)
		SetLED2();
	else
		ClearLED2();

	PS2JumpAddress = PS2ReceiveStart; //Prepare next listening
}






