//********************************
//Special Registers
//********************************

//Common registers
.def d0 = r16
.def d1 = r17
.def d2 = r18
.def d3 = r19
.def d4 = r20
.def d5 = r21




//PortA
.equ MLF = PD4


//PORTB
.equ Motor2b = PB0
.equ Motor1b = PB1
.equ Motor1a = PB2
.equ LED1 = PB7


//PORTC
.equ PS2Data = PC2
.equ PS2Clock = PC3


//PORTD
.equ LED2 = PD5
.equ LED3 = PD6
.equ Motor2a = PD7


//Constants describing a PeripherieTable entry
.equ PT_OFFS_TwiId=0 					// TWI slave ID
.equ PT_OFFS_Name=1 					// textual name
.equ PT_OFFS_PeriId=9					// Unique PeripherieID for hardware: 0..127: Actor, 128..255: Sensor
.equ PT_OFFS_SL_ReadMem_Addr=10			// ReadMemory address in Slave 
.equ PT_OFFS_SL_ReadMem_Length=12		// ReadMemory length (in Bytes) in Slave 
.equ PT_OFFS_SL_WriteMem_Addr=14		// WriteMemory address in Slave 
.equ PT_OFFS_SL_WriteMem_Length=16		// WriteMemory length (in Bytes) in Slave 
.equ PT_OFFS_ReadMem_Addr=18			// Buffer for ReadMemory address in Communicator
.equ PT_OFFS_WriteMem_Addr=20			// Buffer for WriteMemory address in Communicator

.equ PT_OFFS_LOCK=22					// Lock byte
.equ PT_LOCK_TWICurrentWrite_Mask = 0x01	// Bit 0: TWICurrentWrite (Locked state for RS232)
.equ PT_LOCK_RS232CurrentWrite_Mask = 0x02 	// Bit 1: RS232CurrentWrite (Locked state for TWI)
.equ PT_LOCK_NewWriteData_Mask = 0x04		// Bit 2: NewWriteData
.equ PT_LOCK_TWICurrentRead_Mask = 0x08		// Bit 3: TWICurrentRead (Locked state for RS232)
.equ PT_LOCK_RS232CurrentRead_Mask = 0x10	// Bit 4: RS232CurrentRead (Locked state for RS232)

.equ PT_OFFS_TableEntry=23					// Length of one PeripherieTable entry


//***************************************
// Defines Memory access from RS232
//***************************************
Data0ROM:
.db 255, "Jojo1.0 ", 255, low(DataRead), High(DataRead), Low(DataReadEnd-DataRead), High(DataReadEnd-DataRead), Low(DataWrite), High(DataWrite), Low(DataWriteEnd-DataWrite), High(DataWriteEnd-DataWrite ), Low(DataRead), High(DataRead), Low(DataWrite), High(DataWrite), 0 



//*****************************************
//RAM
//*****************************************

.dseg	

.org SRAM_START

Data0:
PeripherieTable: .Byte PT_OFFS_TableEntry // Peripherie Table (Place 0 is reserved for Communicator itself)
PeripherieTable_End:



//************************************************************
// ++++++++++++++Communicators own application memory*********
//************************************************************
DataRead: 
	MouseBuffer: .Byte 256
	LastOffset: .Byte 1
DataReadEnd:

DataWrite: 
	MouseCommand: .Byte 1
DataWriteEnd:


//*******************************************
//*********RS232 Telegramm*******************
//*******************************************
RxDJumpAddress: .Byte 2
RxDActualTelegrammID: .Byte 1
RxDActualCommand: .Byte 1
RxDMemTargetAddress: .Byte 2
RxDPeripherieID: .Byte 1
RxDDataLength: .Byte 2
RxDChecksum: .Byte 1
RxDReadWriteDeviceStatus: .Byte 1 	//Used to signal success of ReadDevice/ WriteDevice commands
.equ MSK_RxD_RWDSError= 0x40		// 0: SUCCEEDED, 0x40: ERROR
TxDJumpAddress: .Byte 2

RS232TimeOutWatchdog: .Byte 1


//**********************************************
//ServoControl
//**********************************************
/*
ServoJumpAddress: .Byte 2
ServoFastPhaseCounter: .Byte 1
Servo0Counter: .Byte 1
Servo1Counter: .Byte 1
Servo2Counter: .Byte 1
Servo3Counter: .Byte 1
*/


//**********************************************
//PS2
//**********************************************
PS2ReceivedByte: .Byte 1
PS2JumpAddress: .Byte 2
PS2SendByte: 	.Byte 1
PS2SendByteChecksum: 	.Byte 1


.cseg
