PS2Init:
	ldi d0, (1<<PCINT11) //Enable Pin change inerrupt on PC3 ()PCINT11)
	sts PCMSK1, d0
	ldi d0, (1<<PCIE1)   //Enable external interrupt for PCINT14:8
	sts PCICR, d0 


	//wipe out MouseCommand
	clr d0
	sts MouseCommand, d0
	sts LastOffset, d0
	

	//wipe out buffer
	clr d0
	clr d1
	SetRamPointer X, MouseBuffer
	st X+, d0
PS2Init_Loop:
	st X+, d0
	dec d1
	brne PS2Init_Loop
PS2Init_Loop_End:

	clr d0
	sts PS2ReceivedByte, d0

	PrepareJump PS2JumpAddress, PS2ReceiveStart

	ret


//Dummy routine for listen disable
PS2ReceiveNop:
	ret


PS2ReceiveStart:
	call SetLED1
	clr d0
	sts PS2ReceivedByte, d0
	PrepareJump PS2JumpAddress, PS2ReceiveBit0
	ret


PS2ReceiveBit0:
	lds d0, PS2ReceivedByte
	sbic PINC, PS2Data
	sbr d0, 0x80
	lsr d0
	sts PS2ReceivedByte, d0
	PrepareJump PS2JumpAddress, PS2ReceiveBit1
	ret

PS2ReceiveBit1:
	lds d0, PS2ReceivedByte
	sbic PINC, PS2Data
	sbr d0, 0x80
	lsr d0
	sts PS2ReceivedByte, d0
	PrepareJump PS2JumpAddress, PS2ReceiveBit2
	ret

PS2ReceiveBit2:
	lds d0, PS2ReceivedByte
	sbic PINC, PS2Data
	sbr d0, 0x80
	lsr d0
	sts PS2ReceivedByte, d0
	PrepareJump PS2JumpAddress, PS2ReceiveBit3
	ret

PS2ReceiveBit3:
	lds d0, PS2ReceivedByte
	sbic PINC, PS2Data
	sbr d0, 0x80
	lsr d0
	sts PS2ReceivedByte, d0
	PrepareJump PS2JumpAddress, PS2ReceiveBit4
	ret

PS2ReceiveBit4:
	lds d0, PS2ReceivedByte
	sbic PINC, PS2Data
	sbr d0, 0x80
	lsr d0
	sts PS2ReceivedByte, d0
	PrepareJump PS2JumpAddress, PS2ReceiveBit5
	ret

PS2ReceiveBit5:
	lds d0, PS2ReceivedByte
	sbic PINC, PS2Data
	sbr d0, 0x80
	lsr d0
	sts PS2ReceivedByte, d0
	PrepareJump PS2JumpAddress, PS2ReceiveBit6
	ret

PS2ReceiveBit6:
	lds d0, PS2ReceivedByte
	sbic PINC, PS2Data
	sbr d0, 0x80
	lsr d0
	sts PS2ReceivedByte, d0
	PrepareJump PS2JumpAddress, PS2ReceiveBit7
	ret

PS2ReceiveBit7:
	lds d0, PS2ReceivedByte
	sbic PINC, PS2Data
	sbr d0, 0x80
	sts PS2ReceivedByte, d0
	call PS2SetReceivedByteToRam
	PrepareJump PS2JumpAddress, PS2ReceiveParity
	ret

PS2ReceiveParity:
	PrepareJump PS2JumpAddress, PS2ReceiveStop
	ret

PS2ReceiveStop:
	PrepareJump PS2JumpAddress, PS2ReceiveStart
	call ClearLED1

	ret




PS2SetReceivedByteToRam:
	//Prepare X register for MouseBuffer+LastOffset access
	ldi XL, low(MouseBuffer)
	ldi XH, high(MouseBuffer)
	lds d0, lastOffset
	clr d1
	add XL, d0
	adc XH, d1

	//Transfer byte from PS2ReceivedByte to MouseBuffer+LastOffset
	lds d0, PS2ReceivedByte
	st X, d0

	// Increment LastOffset
	lds d0, LastOffset
	inc d0
	sts LastOffset, d0
	ret





;******************************************
;External PCINT PinChange Interrupt
;******************************************
PCINT1_Interrupt:
	;Register retten
	push d0
	in d0, SREG
	push d0
	push d1
	push d2
	push d3
	push XL
	push XH
	push YL
	push YH
	push ZL
	push ZH

	call PCINT1_InterruptService
	
	;Register zurckholen
	pop ZH
	pop ZL
	pop YH
	pop YL
	pop XH
	pop XL
	pop d3
	pop d2
	pop d1
	pop d0
	out SREG, d0	
	pop d0
	reti



PCINT1_InterruptService:
	sbic PINC, PS2Clock
	ret
	JUMP PS2JumpAddress





//******************************************************************************************************************
//SEND
//******************************************************************************************************************
PS2StartSend:
	//Revceiver off
	PrepareJump PS2JumpAddress, PS2ReceiveNop

	//Prepare checksum
	lds d0, PS2SendByte
	call MakechecksumOfByte
	sts PS2SendByteChecksum, d1
		

	// pull down Clock for 100u
	call PS2PullDownClock
	call wait100u

	// initialize Hump address
	PrepareJump PS2JumpAddress, PS2SendBit0
	
	// pulldown data and release clock
	call PS2PullDownData
	call PS2ReleaseClock
	ret
	


//Basic Port functions
PS2PullDownClock:
	cbi PORTC, PS2Clock
	sbi DDRC, PS2Clock
	ret

PS2ReleaseClock:
	cbi DDRC, PS2Clock
	ret


PS2PullDownData:
	cbi PORTC, PS2Data
	sbi DDRC, PS2Data
	ret


PS2ReleaseData:
	cbi DDRC, PS2Data
	ret






//Send states
PS2SendBit0:
	lds d0, PS2SendByte
	call PS2ReleaseData
	sbrs d0, 0
	call PS2PullDownData
	PrepareJump PS2JumpAddress, PS2SendBit1
	ret


PS2SendBit1:
	lds d0, PS2SendByte
	call PS2ReleaseData
	sbrs d0, 1
	call PS2PullDownData
	PrepareJump PS2JumpAddress, PS2SendBit2
	ret

PS2SendBit2:
	lds d0, PS2SendByte
	call PS2ReleaseData
	sbrs d0, 2
	call PS2PullDownData
	PrepareJump PS2JumpAddress, PS2SendBit3
	ret

PS2SendBit3:
	lds d0, PS2SendByte
	call PS2ReleaseData
	sbrs d0, 3
	call PS2PullDownData
	PrepareJump PS2JumpAddress, PS2SendBit4
	ret

PS2SendBit4:
	lds d0, PS2SendByte
	call PS2ReleaseData
	sbrs d0, 4
	call PS2PullDownData
	PrepareJump PS2JumpAddress, PS2SendBit5
	ret

PS2SendBit5:
	lds d0, PS2SendByte
	call PS2ReleaseData
	sbrs d0, 5
	call PS2PullDownData
	PrepareJump PS2JumpAddress, PS2SendBit6
	ret

PS2SendBit6:
	lds d0, PS2SendByte
	call PS2ReleaseData
	sbrs d0, 6
	call PS2PullDownData
	PrepareJump PS2JumpAddress, PS2SendBit7
	ret

PS2SendBit7:
	lds d0, PS2SendByte
	call PS2ReleaseData
	sbrs d0, 7
	call PS2PullDownData
	PrepareJump PS2JumpAddress, PS2SendBitParity
	ret

PS2SendBitParity:
	lds d0, PS2SendByteChecksum
	call PS2ReleaseData
	sbrc d0, 0
	call PS2PullDownData
	PrepareJump PS2JumpAddress, PS2SendBitStop
	ret


PS2SendBitStop:
	call PS2ReleaseData
	PrepareJump PS2JumpAddress, PS2Send_ReceiveAck
	
	ret


PS2Send_ReceiveAck:
	call ClearLED2
	sbic PINC, PS2Data
	call SetLED2
	PrepareJump PS2JumpAddress, PS2ReceiveStart //Prepare next listening

	ret







