//***********************************************************************
// Decrements content of 16 Bit counter in RAM @0
// Returns in d0:
// 0: Counter is already 0 and cannot be decremented
// FF: counter is decremented
// Remarks:
// Decrement16BitRamCounterUntilNull below returns 0 in d0 
//		one time earlier than this macro.
//***********************************************************************
.MACRO Mac_Decrement16BitRamCounterIfNotNull
	push YL
	push YH
	push d1
	ldi YL, Low(@0)
	ldi YH, High(@0)

	call Decrement16BitRamCounterIfNotNull
	
	pop d1		
	pop YH
	pop YL

.ENDMACRO


// Test address for 2 Byte RAM counter assumed in X
// Ram counter <> 0: Ram counter will be decremented, d0 returns 0xFF
// Ram counter = 0: d0 returns 0
Decrement16BitRamCounterIfNotNull:
	ld d0, Y
	ldd d1, Y+1
	cpi d0, 0
	brne Decrement16BitRamCounterIfNotNull_LowerNot0
	cpi d1, 0
	brne Decrement16BitRamCounterIfNotNull_UpperNot0
Decrement16BitRamCounterIfNotNull_Is0:
	ldi d0, 0 // Return value: Counter == 0
	ret		
Decrement16BitRamCounterIfNotNull_UpperNot0:
	dec d1
	std Y+1, d1
	dec d0
	st Y, d0
	ldi d0, 0xFF // Return value: Counter <> 0
	ret
Decrement16BitRamCounterIfNotNull_LowerNot0:
	dec d0
	st Y, d0
	ldi d0, 0xFF // Return value: Counter <> 0
Decrement16BitRamCounterIfNotNull_End:
	ret	


//***********************************************************************
// Decrements content of 16 Bit counter in RAM @0
// Returns in d0:
// 0: Counter is set to 0 (in earlier loops)
// FF: counter is not 0 
//***********************************************************************
.MACRO Mac_Decrement16BitRamCounterUntilNull
	push XH
	push XL
	push d1
	ldi d0, 0xFF
	lds XL, @0
	lds XH, @0+1
	sbiw X, 1
	in d1, SREG
	sbrc d1, SREG_Z
	ldi d0, 0x00
	sts @0, XL
	sts @0+1, XH

	pop d1
	pop XL
	pop XH		

.ENDMACRO 

//***********************************************************************
.MACRO Mac_Increment16BitRAMCounter
	push d0
	push d1
	push d2
	lds d1, @0+1
	lds d0, @0
	inc d0
	in d2, SREG
	sbrc d2, SREG_Z
	inc d1
	sts @0, d0
	sts @0+1, d1
	pop d2
	pop d1
	pop d0
.ENDMACRO
	


//***********************************************************************
//Returns in d0:
// 	0: 	d1/d0 = d3/d2
// 	1: 	d1/d0 > d3/d2
// 	FF: d1/d0 < d3/d2
//***********************************************************************
Compare16BitRegisters:
	cp d1, d3
	breq Compare16BitRegisters_LowerByte
	brsh Compare16BitRegisters_IsHigher
	ldi d0, 0xFF
	ret

Compare16BitRegisters_IsHigher:
	ldi d0, 0x01
	ret

Compare16BitRegisters_IsSame:
	clr d0
	ret

Compare16BitRegisters_LowerByte:
	cp d0, d2
	breq Compare16BitRegisters_IsSame
	brsh Compare16BitRegisters_IsHigher
	ldi d0, 0xFF
	ret


