//Interrupt vector table
; ***** INTERRUPT VECTORS ************************************************
.cseg					;CODE segment
.org 0					



jmp Init  						;Reset

jmp UNUSED_INT 		; External Interrupt Request 0
jmp UNUSED_INT 		; External Interrupt Request 1
jmp UNUSED_INT 		; Pin Change Interrupt Request 0
jmp PCINT1_Interrupt; Pin Change Interrupt Request 1
jmp UNUSED_INT 		; Pin Change Interrupt Request 2
jmp UNUSED_INT 		; Watchdog Time-out Interrupt
jmp UNUSED_INT 		; Timer/Counter2 Compare Match A
jmp UNUSED_INT 		; Timer/Counter2 Compare Match B
jmp UNUSED_INT 		; Timer/Counter2 Overflow
jmp UNUSED_INT 		; Timer/Counter1 Capture Event
jmp UNUSED_INT		; Timer/Counter1 Compare Match A
jmp UNUSED_INT 		; Timer/Counter1 Compare Match B
jmp UNUSED_INT 		; Timer/Counter1 Overflow
jmp UNUSED_INT  	; TimerCounter0 Compare Match A
jmp UNUSED_INT 		; TimerCounter0 Compare Match B
jmp MainTimerInterrupt 		; Timer/Couner0 Overflow
jmp UNUSED_INT 		; SPI Serial Transfer Complete
jmp RxD_Interrupt		; USART Rx Complete
jmp UNUSED_INT 		; USART, Data Register Empty
jmp TxD_Interrupt		; USART Tx Complete
jmp UNUSED_INT 		; ADC Conversion Complete
jmp UNUSED_INT 		; EEPROM Ready
jmp UNUSED_INT 		; Analog Comparator
jmp UNUSED_INT 		; Two-wire Serial Interface
jmp UNUSED_INT 		; Store Program Memory Read

#include "m168def.inc"
#include "Constants.asm"
#include "Macros.asm"
#include "SixteenBits.asm"
#include "Waiters.asm"
#include "rs232.asm"
#include "rs232Telegram.asm"
#include "LED.asm"
#include "PS2.asm"
#include "MainTimer.asm"









//Formal unused Interrupt target address to enlarge vector table entries to 32 bit
UNUSED_INT:
	reti

Init:
	//Stack pointer
	ldi d0, High(RAMEND)
	out SPH, d0
	ldi d0, Low(RAMEND)
	out SPL, d0

	//PORTB
	ldi d0, (1<<Motor2b) + (1<<Motor1b) + (1<<Motor1a) + (1<<LED1)
	out DDRB, d0
	clr d0
	out PORTB, d0

	//PortC
	ldi d0, 0 
	out DDRC, d0
	clr d0
	out PORTC, d0



	//PORTD
	ldi d0, (1<<LED2)+(1<<LED3)+(1<<Motor2a)
	out DDRD, d0
	ldi d0 , 0
	out PORTD, d0


	call InitData0
	call InitRS232Telegram
	call INIT_COM
	call InitMainTimer

	call PS2Init
	sei


Main:
	//jmp Main
	lds d0, MouseCommand
	cpi d0, 0 
	breq Main
	//Something to send
	sts PS2SendByte, d0
	clr d0
	sts MouseCommand, d0
	call PS2StartSend
	rjmp Main




//************************************************************************************
//InitData0:
// Inits Data0 RAM From ROM
// A little bit tricky: Data0 and PeripherieTable share the same address..
//************************************************************************************
InitData0:
	ldi ZH, High(Data0ROM << 1)
	ldi ZL, Low(Data0ROM << 1)
	ldi XH, High(Data0)
	ldi XL, Low(Data0)
	ldi d1, PT_OFFS_TableEntry

InitData0Loop:
	lpm d0, Z+
	st X+, d0
	dec d1
	brne InitData0Loop
InitData0_End:
	ret





